/*
 *+--------------------------------------------------------------------------------------+
 *| Licensed Materials - Use restricted, please refer to the "Samples Gallery" terms and |
 *| conditions in the IBM International Program License Agreement.						 |	
 *|  Copyright IBM Corporation 2003, 2004. All Rights Reserved.						 |
 *+--------------------------------------------------------------------------------------+
 */
package itso.ad.common.util;

import javax.ejb.EJBHome;
import javax.ejb.EJBLocalHome;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import itso.ad.common.logging.LogHelper;

public class HomeFactory {	
	
	private static LogHelper log = LogHelper.getInstance(HomeFactory.class);
	
	private static HomeFactory homefactory;
	private InitialContext initialContext = null;
	
	protected HomeFactory() {
		super();
		try {
			initialContext = new InitialContext();
		} catch (NamingException namingException) {
			log.error(namingException.getMessage(), namingException);
		}
	}
	
	static public HomeFactory getInstance() {
		if (homefactory == null) {
			homefactory = new HomeFactory();
		}
	
		return homefactory;
	}
	
	public Object getHome(String ejbRef) throws NamingException {
	
		if (initialContext != null) {
	
			Object nsObject = initialContext.lookup(new StringBuffer("java:comp/env/").append(ejbRef).toString());
			//Object nsObject = initialContext.lookup(ejbRef);
	
			if (log.isDebugEnabled()) {
				log.debug("class=" + nsObject.getClass());
			}
	
			if (nsObject instanceof EJBLocalHome) {
				if (log.isDebugEnabled()) {
					log.debug("ejbRef " + ejbRef + " is a local ref.");
				}
				return nsObject;
			} else {
				EJBHome ejbHome = (EJBHome) javax.rmi.PortableRemoteObject.narrow((org.omg.CORBA.Object) nsObject, EJBHome.class);
	
				if (log.isDebugEnabled()) {
					log.debug("ejbRef " + ejbRef + " is a remote ref.");
				}
				return ejbHome;
			}
		} else {
			throw new NamingException("HomeFactory:no InitialContext");
		}
	}
}