/*
 *+--------------------------------------------------------------------------------------+
 *| Licensed Materials - Use restricted, please refer to the "Samples Gallery" terms and |
 *| conditions in the IBM International Program License Agreement.
 *| (C) Copyright IBM Corporation 2003, 2004. All Rights Reserved.						 |	
 *+--------------------------------------------------------------------------------------+
 */

package itso.ad.business.ejb.facade;

import java.lang.String;
import java.lang.Integer;
import java.util.List;
import itso.ad.business.framework.interfaces.to.ICustomerTO;

import itso.ad.business.ejb.model.AccountKey;
import itso.ad.business.ejb.model.AccountLocal;
import itso.ad.business.ejb.model.AccountLocalHome;
import itso.ad.business.ejb.model.CustomerKey;
import itso.ad.business.ejb.model.CustomerLocal;
import itso.ad.business.ejb.model.CustomerLocalHome;
import itso.ad.business.framework.exception.ServiceException;
import itso.ad.business.framework.interfaces.to.IAccountTO;
import itso.ad.citybank.dao.CityBankDataAccessObject;
import itso.ad.common.logging.LogHelper;
import itso.ad.common.util.HomeFactory;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.ejb.CreateException;
import javax.ejb.DuplicateKeyException;
import javax.ejb.EJBLocalHome;
import javax.ejb.FinderException;
import javax.naming.NamingException;


/**
 * Bean implementation class for Enterprise Bean: PiggyBankController
 */
public class PiggyBankControllerBean implements javax.ejb.SessionBean {
	private static LogHelper log = LogHelper.getInstance(PiggyBankControllerBean.class);

	private static final String ACCOUNT_EJB_REF = "ejb/Account";
	private static final String CUSTOMER_EJB_REF = "ejb/Customer";
	//private static final String ACCOUNT_EJB_REF = "ejb/itso/ad/business/ejb/model/AccountHome";
	//private static final String CUSTOMER_EJB_REF = "ejb/itso/ad/business/ejb/model/CustomerHome";

	
	private javax.ejb.SessionContext mySessionCtx;
	/**
	 * getSessionContext
	 */
	public javax.ejb.SessionContext getSessionContext() {
		return mySessionCtx;
	}
	/**
	 * setSessionContext
	 */
	public void setSessionContext(javax.ejb.SessionContext ctx) {
		mySessionCtx = ctx;
	}
	/**
	 * @param customerId
	 * @return
	 * @throws ServiceException
	 * @throws RemoteException
	 * @!generated "UML to EJB (uml2.ejb.transformation)"
	 */
	public ICustomerTO getCustomerById(String customerId) throws ServiceException
 {
		ICustomerTO customerTO = null;

		try {
			CustomerLocalHome custHome = (CustomerLocalHome) getHome(CUSTOMER_EJB_REF);
			CustomerLocal cust = custHome.findByPrimaryKey(new CustomerKey(customerId));
			customerTO = cust.getData();
			

		} catch (FinderException e) {
			log.error(e.getMessage(), e);
			throw new ServiceException("Customer: " + customerId + " not found", "error.customer.notfound", e);
		} catch (NamingException e) {
			log.error(e.getMessage(), e);
			throw new ServiceException("ejb ref: " + CUSTOMER_EJB_REF + " not found", e);
		}

		return customerTO;
	}

	/**
	 * @param customerId
	 * @param debitAccountNr
	 * @param creditAccountNr
	 * @param amountToTransfer
	 * @throws ServiceException
	 * @!generated "UML to EJB (uml2.ejb.transformation)"
	 */
	public void transfer(String customerId, String debitAccountNr,
			String creditAccountNr, Integer amountToTransfer) throws ServiceException{

		try {
			AccountLocalHome accountHome = (AccountLocalHome) getHome(ACCOUNT_EJB_REF);
			AccountLocal debitAccount=null;
			AccountLocal creditAccount=null;
			
			try {
				debitAccount = accountHome.findByPrimaryKey(new AccountKey(debitAccountNr));
			} catch (FinderException e) {
				throw new ServiceException("Debit account: " + debitAccountNr + " not found", "error.debit.notfound", e);
			}

			try {
				creditAccount = accountHome.findByPrimaryKey(new AccountKey(creditAccountNr));
			} catch (FinderException e) {
				throw new ServiceException("Credit account: " + creditAccountNr + " not found", "error.credit.notfound", e);
			}

			IAccountTO debitAccountData = debitAccount.getData();

			Integer debitAccountBalance = debitAccountData.getBalance();

			if (amountToTransfer.intValue() > debitAccountBalance.intValue()) {
				throw new ServiceException("Account transfer failed, the maximum transfer rate is: " + debitAccountBalance, "error.account.insufficientfunds");
			}

			debitAccount.setBalance(new Integer(debitAccountBalance.intValue() - amountToTransfer.intValue()));
			IAccountTO creditAccountData = creditAccount.getData();
			creditAccount.setBalance(new Integer(creditAccountData.getBalance().intValue() + amountToTransfer.intValue()));
		} catch (ServiceException e) {
			log.error(e.getMessage(), e);
			getSessionContext().setRollbackOnly(); // force rollback
			throw e;
		} catch (NamingException e) {
			log.error(e.getMessage(), e);
			throw new ServiceException("ejb ref: " + ACCOUNT_EJB_REF + " not found", e);
		}
	}

	/**
	 * @param checkReference
	 * @param checkAmount
	 * @param accountToCredit
	 * @!generated "UML to EJB (uml2.ejb.transformation)"
	 */
	public void cashCheck(String checkReference, Integer checkAmount,
			String accountToCredit) throws ServiceException
	{
			
			// validate the check against CityBank
			boolean checkValid = false;
			try {			
				CityBankDataAccessObject dao = new CityBankDataAccessObject();
				checkValid = dao.checkCityBankAccount(checkReference, checkAmount.intValue());
			} catch (Exception e) {  // DataAccessException
				log.error(e.getMessage());
				throw new ServiceException(e.getMessage(), "error.connect.citybank", e);
			}
			if (!checkValid) {
				throw new ServiceException("Check " + checkReference + " with amount " + checkAmount + " is not valid", "error.check.invalid");
			}

			// credit account
			try {
				AccountLocalHome accountHome = (AccountLocalHome) getHome(ACCOUNT_EJB_REF);
				AccountLocal creditAccount = accountHome.findByPrimaryKey( new AccountKey(accountToCredit));
				creditAccount.setBalance(new Integer(creditAccount.getData().getBalance().intValue() + checkAmount.intValue()));
			} catch (Exception e) {
				log.error(e.getMessage());
				throw new ServiceException(e.getMessage(), "error.credit.account", e);
			}
			
	}

	/**
	 * @param customerId
	 * @return
	 * @throws ServiceException
	 * @!generated "UML to EJB (uml2.ejb.transformation)"
	 */
	public List getAccountsByCustomerId(String customerId) throws ServiceException{
		List accountList = null;

		try {
			CustomerLocalHome customerHome = (CustomerLocalHome) getHome(CUSTOMER_EJB_REF);
			CustomerLocal customer = customerHome.findByPrimaryKey(new CustomerKey(customerId));

			Collection accounts = customer.getAccounts();

			if (accounts != null && !accounts.isEmpty()) {

				accountList = new ArrayList();
				Iterator iterator = accounts.iterator();

				while (iterator.hasNext()) {
					AccountLocal element = (AccountLocal) iterator.next();
					accountList.add(element.getData());
				}
			} else {
				throw new ServiceException("No accounts for: " + customerId + " found", "error.customer.noaccounts");
			}
		} catch (FinderException e) {
			log.error(e.getMessage(), e);
			throw new ServiceException("Customer: " + customerId + " not found", "error.customer.notfound", e);
		} catch (NamingException e) {
			log.error(e.getMessage(), e);
			throw new ServiceException("ejb ref: " + CUSTOMER_EJB_REF + " not found", e);
		}

		return accountList;
	}

	/**
	 * @param customerId
	 * @param accountNumber
	 * @return
	 * @throws ServiceException
	 * @!generated "UML to EJB (uml2.ejb.transformation)"
	 */
	public Integer getBalance(String customerId, String accountNumber) throws ServiceException {
		Integer balance;

		try {
			AccountLocalHome accountHome = (AccountLocalHome) getHome(ACCOUNT_EJB_REF);
			AccountLocal account = accountHome.findByPrimaryKey(new AccountKey(accountNumber));

			IAccountTO accountData = account.getData();
			balance = accountData.getBalance();
		} catch (FinderException e) {
			log.error(e.getMessage(), e);
			throw new ServiceException("Account: " + accountNumber + " not found", "error.account.notfound", e);
		} catch (NamingException e) {
			log.error(e.getMessage(), e);
			throw new ServiceException("ejb ref: " + ACCOUNT_EJB_REF + " not found", e);
		}

		return balance;
	}
	
	/**
	 * @param customerId
	 * @param accountNumber
	 * @return
	 * @generated "UML to EJB (uml2.ejb.transformation)"
	 */
	public void createNewCustomer(String customerId, String customerName, String accountNumber)
	throws ServiceException
	{
		CustomerLocal customerLocal= null;

			CustomerLocalHome custHome=null;
			try {
				custHome = (CustomerLocalHome) getHome(CUSTOMER_EJB_REF);
			} catch (NamingException e1) {
				
				e1.printStackTrace();
				throw new ServiceException("ejb ref: " + CUSTOMER_EJB_REF + " not found", e1);
			}
			try {
				customerLocal = custHome.create(customerId);
				customerLocal.setName(customerName);
			}catch (DuplicateKeyException de)
			{
				try {
					customerLocal = custHome.findByPrimaryKey(new CustomerKey(customerId));
				} catch (FinderException e4) {
					
					e4.printStackTrace();
					throw new ServiceException("Customer: " + customerId + " already exist or database error", "error.customer.creation", e4);
				}
				
			} 
			catch (CreateException e) {
				
				e.printStackTrace();
				throw new ServiceException("Customer: " + customerId + " already exist or database error", "error.customer.exist", e);
			}
			
			AccountLocalHome accountHome = null;
			AccountLocal account=null;
			try {
				accountHome = (AccountLocalHome) getHome(ACCOUNT_EJB_REF);
			} catch (NamingException e2) {
				e2.printStackTrace();
				throw new ServiceException("ejb ref: " + ACCOUNT_EJB_REF + " not found", e2);
			}
			try {
				account = accountHome.findByPrimaryKey(new AccountKey(accountNumber));
			} catch (FinderException e3) {
				//e3.printStackTrace();
				try {
					///the account may not be exist, let try to create a new one
					account = accountHome.create(accountNumber);
				} catch (CreateException e4) {
					e4.printStackTrace();
					throw new ServiceException("Account: " + accountNumber + " unable to create an account, may be database error", "error.account.unable to create", e4);
				}
			}
			account.setTheCustomer(customerLocal);
			
	}

	/**
	 * ejbCreate
	 */
	public void ejbCreate() throws javax.ejb.CreateException {
	}

	/**
	 * ejbActivate
	 */
	public void ejbActivate() {
	}

	/**
	 * ejbPassivate
	 */
	public void ejbPassivate() {
	}

	/**
	 * ejbRemove
	 */
	public void ejbRemove() {
	}
	
	/**
	 * Return a local ejb home.
	 * 
	 * @param ejbRef <code>String</code> the ejb reference of the sought bean.
	 * @return <code>EJBLocalHome</code> the ejb local home reference of the sought been.
	 * 
	 * @exception NamingException if the lookup failed.
	 */
	protected EJBLocalHome getHome(String ejbRef) throws NamingException {
		return (EJBLocalHome) HomeFactory.getInstance().getHome(ejbRef);
	}
}